#include <dlib/opencv.h> 
#include <dlib/image_processing.h> 
#include <dlib/image_processing/frontal_face_detector.h> 

#include <opencv2/opencv.hpp> 

using namespace dlib; 
using namespace std; 
using namespace cv; 

#define faceWidth 64
#define faceHeight 64
#define PI 3.14159265

void alignFace(Mat &imFace, Mat &alignedImFace, std::vector<Point2f> &landmarks) 
{
    float l_x = landmarks[39].x;
    float l_y = landmarks[39].y;
    float r_x = landmarks[42].x;
    float r_y = landmarks[42].y;
    float dx = r_x - l_x;
    float dy = r_y - l_y;
    double angle = atan2(dy, dx) * 180 / PI;
    Point2f eyesCenter;
    eyesCenter.x = (l_x + r_x) / 2.0;
    eyesCenter.y = (l_y + r_y) / 2.0;
    Mat rotMatrix = Mat(2, 3, CV_32F);
    rotMatrix = getRotationMatrix2D(eyesCenter, angle, 1);
    warpAffine(imFace, alignedImFace, rotMatrix, imFace.size());
}

bool rectAreaComparator(dlib::rectangle &r1, dlib::rectangle &r2)
{ 
    return r1.area() < r2.area(); 
}

std::vector<Point2f> getLandmarks (frontal_face_detector &faceDetector, shape_predictor &landmarkDetector, Mat &img, float FACE_DOWNSAMPLE_RATIO = 1)
{
    std::vector<Point2f> points;
    Mat imgSmall;
    cv::resize(img, imgSmall, cv::Size(), 1.0/FACE_DOWNSAMPLE_RATIO, 1.0/FACE_DOWNSAMPLE_RATIO);
    dlib::cv_image<dlib::bgr_pixel> dlibIm(img);
    dlib::cv_image<dlib::bgr_pixel> dlibImSmall(imgSmall);
    std::vector<dlib::rectangle> faceRects = faceDetector(dlibImSmall);
    if (faceRects.size() > 0) {
        dlib::rectangle rect = *std::max_element(faceRects.begin(), faceRects.end(), rectAreaComparator);
        dlib::rectangle scaledRect(
            (long)(rect.left() * FACE_DOWNSAMPLE_RATIO),
            (long)(rect.top() * FACE_DOWNSAMPLE_RATIO),
            (long)(rect.right() * FACE_DOWNSAMPLE_RATIO),
            (long)(rect.bottom() * FACE_DOWNSAMPLE_RATIO)
        );
        dlib::full_object_detection landmarks = landmarkDetector(dlibIm, scaledRect);
        for (size_t i = 0; i < landmarks.num_parts(); i++) {
            Point2f pt(landmarks.part(i).x(), landmarks.part(i).y());
            points.push_back(pt);
        }
    }
    return points;
}

Mat getCroppedFaceRegion(Mat image, std::vector<Point2f> landmarks)
{
    int x1Limit = landmarks[0].x - (landmarks[36].x - landmarks[0].x);
    int x2Limit = landmarks[16].x + (landmarks[16].x - landmarks[45].x);
    int y1Limit = landmarks[27].y - 3*(landmarks[30].y - landmarks[27].y);
    int y2Limit = landmarks[8].y + (landmarks[30].y - landmarks[29].y);
    int x1 = max(x1Limit,0);
    int x2 = min(x2Limit, image.cols);
    int y1 = max(y1Limit, 0);
    int y2 = min(y2Limit, image.rows);
    cv::Rect selectedRegion = cv::Rect( x1, y1, x2-x1, y2-y1 );
    return image(selectedRegion);
}

