#include "listdir.h"

#include <dlib/opencv.h> 
#include <dlib/image_processing.h> 
#include <dlib/image_processing/frontal_face_detector.h> 
 
#include <opencv2/opencv.hpp> 
#include <opencv2/face.hpp> 

#include <sstream> 
#include <fstream> 

#include "Preprocessing.h"
 
using namespace dlib; 
using namespace std; 
using namespace cv; 

int main (int argc, char* argv[]) 
{ 
	frontal_face_detector faceDetector = get_frontal_face_detector();
	shape_predictor landmarkDetector;
	deserialize("../../dlib-19.13/data/shape_predictor_68_face_landmarks.dat") >> landmarkDetector;
	
    string faceDatasetFolder = "../../dlib-19.13/data/faces";
    Ptr<face::FaceRecognizer> faceRecognizer;
    faceRecognizer = face::LBPHFaceRecognizer::create();
    faceRecognizer->read(faceDatasetFolder+"/model.yml");

    std::vector<string> labels;
    listdirs(faceDatasetFolder, labels);
    printdirs(labels);

    VideoCapture camera(0);

    for (;;) {
        Mat frame;
        camera >> frame;

        std::vector<Point2f> landmarks = getLandmarks(faceDetector, landmarkDetector, frame);
        landmarks = stabilize(landmarks,5);
        if (landmarks.size() < 68) continue;

        Mat im;
        cvtColor(frame, im, COLOR_BGR2GRAY);
        Mat imFace = getCroppedFaceRegion(im, landmarks);
        Mat alignedImFace;
        alignFace(imFace, alignedImFace, landmarks);
		cv::resize(alignedImFace, alignedImFace, Size(faceHeight, faceWidth));
        alignedImFace.convertTo(alignedImFace, CV_32F, 1.0/255);

        int prediction = -1;
        double score = 0.0;
        faceRecognizer->predict(alignedImFace, prediction, score);
        prediction = stabilize(prediction,5);

        for (size_t i=0; i<landmarks.size(); i++) circle(frame,landmarks[i],2,Scalar(0,0,255),CV_FILLED);
        if (prediction >= 0) putText(frame, labels[prediction], landmarks[19]-cv::Point2f(0,18), FONT_HERSHEY_PLAIN, 1.0, Scalar(255, 0, 0), 2);
        imshow("frame",frame);

        int key = waitKey(30);
        if (key == 27) break;
    }

	return 0; 
} 
 
