#include "createpath.h"
#include "listdir.h"

#include <dlib/opencv.h> 
#include <dlib/image_processing.h> 
#include <dlib/image_processing/frontal_face_detector.h> 
 
#include <opencv2/opencv.hpp> 
#include <opencv2/face.hpp> 

#include <sstream> 
#include <fstream> 
#include <time.h>

#include "Preprocessing.h"
#include "Training.h"
 
using namespace dlib; 
using namespace std; 
using namespace cv; 

int main (int argc, char* argv[]) 
{ 
	frontal_face_detector faceDetector = get_frontal_face_detector();
	shape_predictor landmarkDetector;
	deserialize("../../dlib-19.13/data/shape_predictor_68_face_landmarks.dat") >> landmarkDetector;
	
    //Ptr<face::FaceRecognizer> faceRecognizer;
    //faceRecognizer = face::LBPHFaceRecognizer::create();
    //faceRecognizer->read(faceDatasetFolder+"/model.yml");
    training_prepare(faceDetector,landmarkDetector);
    Ptr<face::FaceRecognizer> faceRecognizer = training_perform();

    std::vector<string> labels;
    listdirs(faceDatasetFolder, labels);
	int label = -1;

    VideoCapture camera(0);

    for (;;) {
        Mat frame;
        camera >> frame;

        std::vector<Point2f> landmarks = getLandmarks(faceDetector, landmarkDetector, frame);
        if (landmarks.size() < 68) continue;

        Mat im;
        cvtColor(frame, im, COLOR_BGR2GRAY);
        Mat imFace = getCroppedFaceRegion(im, landmarks);
        Mat alignedImFace;
        alignFace(imFace, alignedImFace, landmarks);
		cv::resize(alignedImFace, alignedImFace, Size(faceHeight, faceWidth));
        alignedImFace.convertTo(alignedImFace, CV_32F, 1.0/255);

        int prediction = -1;
        double score = 0.0;
        faceRecognizer->predict(alignedImFace, prediction, score);

        for (size_t i=0; i<landmarks.size(); i++) circle(frame,landmarks[i],2,Scalar(0,0,255),CV_FILLED);
		
		Mat view = frame.clone();
        if (prediction >= 0) putText(view, labels[prediction], landmarks[19]-cv::Point2f(0,16), FONT_HERSHEY_PLAIN, 0.9, Scalar(255,0,0), 2);
		if (label != -1 && prediction != -1 && label != prediction) {
			view.setTo(Scalar(0,0,0));
			putText(view, "Training", cv::Point(view.cols/2-25,view.rows/2), FONT_HERSHEY_PLAIN, 2.0, Scalar(255,255,255), 3);
		}
        imshow("frame",view);

        int key = waitKey(30);
        if (key == 27) break;
		else if (key == 'i') {
			cout << "Enter label of your name: " << endl;
			printdirs(labels);
            cout << labels.size() << ". NEW NAME" << endl;
			cin >> label;
            if (label == labels.size()) {
                cout << "Enter your name: ";
                string name;
                cin >> name;
                createpath(faceDatasetFolder+"/"+name);
                labels.push_back(name);
            }
			if (label < 0 || label >= labels.size()) label = -1;
			else cout << endl << "Current name: " << labels[label] << endl;
			if (label == -1) cout << endl << "No current name" << endl;	
		}
		else if (key == 'o') {
			label = -1;
			cout << endl << "No current name" << endl;			
		}
		
        if (label != -1 && prediction != -1 && label != prediction) {
            imwrite(faceDatasetFolder+"/"+labels[label]+"/"+to_string(time(NULL))+".jpg",frame);
            training_update(alignedImFace,label);
            faceRecognizer = training_perform();
        }
    }

	return 0; 
} 
 
