#include "listdir.h"

#include <dlib/opencv.h> 
#include <dlib/image_processing.h> 
#include <dlib/image_processing/frontal_face_detector.h> 
 
#include <opencv2/opencv.hpp> 
#include <opencv2/face.hpp>

#include <sstream> 
#include <fstream> 

#include "Preprocessing.h"
 
using namespace dlib; 
using namespace std; 
using namespace cv; 

static std::vector<Mat> imagesFaceTrain;
static std::vector<int> labelsFaceTrain;
static std::string faceDatasetFolder = "../../dlib-19.13/data/faces";

void training_prepare (frontal_face_detector &faceDetector, shape_predictor &landmarkDetector) 
{ 
	std::vector<string> folders;
	std::vector<std::vector<string>> filenames;
	listdirs(faceDatasetFolder, folders, filenames);
    printdirs(folders,filenames);
    cout << "Preparing ..." << endl;

    for (size_t i=0; i<folders.size(); i++) {
        for (size_t j=0; j<filenames[i].size(); j++) {
            Mat im = cv::imread(faceDatasetFolder+"/"+folders[i]+"/"+filenames[i][j]);
            std::vector<Point2f> landmarks = getLandmarks(faceDetector, landmarkDetector, im);
            if (landmarks.size() < 68) continue;

            cvtColor(im, im, COLOR_BGR2GRAY);
            Mat imFace = getCroppedFaceRegion(im, landmarks);

            Mat alignedImFace;
            alignFace(imFace, alignedImFace, landmarks);
            cv::resize(alignedImFace, alignedImFace, Size(faceHeight, faceWidth));
            alignedImFace.convertTo(alignedImFace, CV_32F, 1.0/255);

            imagesFaceTrain.push_back(alignedImFace);    
            labelsFaceTrain.push_back(i);
        }
    }
}

void training_update(Mat &alignedImFace, int label)
{
    imagesFaceTrain.push_back(alignedImFace);    
    labelsFaceTrain.push_back(label);
}

Ptr<face::FaceRecognizer> training_perform ()
{
    cout << "Training ..." << endl;
    Ptr<face::FaceRecognizer> faceRecognizerLBPH = face::LBPHFaceRecognizer::create();
    faceRecognizerLBPH->train(imagesFaceTrain, labelsFaceTrain);
    faceRecognizerLBPH->write(faceDatasetFolder+"/model.yml");
    cout << "... model saved" << endl;
	return faceRecognizerLBPH;
} 
 
